@@ -9,12 +9,9 @@
 #' @seealso \code{\link{Rase}}.
 #' @examples
 #' set.seed(0, kind = "L'Ecuyer-CMRG")
-#' train.data <- RaModel(1, n = 100, p = 50)
-#' test.data <- RaModel(1, n = 100, p = 50)
+#' train.data <- RaModel("classification", 1, n = 100, p = 50)
 #' xtrain <- train.data$x
 #' ytrain <- train.data$y
-#' xtest <- test.data$x
-#' ytest <- test.data$y
 #'
 #' # test RaSE classifier with LDA base classifier
 #' fit <- Rase(xtrain, ytrain, B1 = 50, B2 = 50, iteration = 0, cutoff = TRUE,
@@ -30,6 +27,7 @@ print.RaSE <- function(x, ...) {
   cat("Criterion:", x$criterion, "\n")
   cat("B1:", x$B1, "\n")
   cat("B2:", x$B2, "\n")
+  cat("D:", x$D, "\n")
   cat("Cutoff:", x$cutoff, "\n")
   if (!is.null(x$ranking)) {
     cat("Selected percentage of each feature appearing in B1 subspaces:", "\n")
